@@ -1,12 +1,11 @@
-from .Sequencer import Sequencer
-from .Modeller import Modeller
-from .Documenter import Documenter
-from .DataSampler import DataSampler
+#  Copyright (c) 2022 by Amplo.
 from .DataExplorer import DataExplorer
-from .DataProcessor import DataProcessor
-from .DataProcessor import DataProcesser
+from .DataProcessor import DataProcesser, DataProcessor
+from .DataSampler import DataSampler
+from .Documenter import Documenter
 from .DriftDetector import DriftDetector
-from .FeatureProcessor import FeatureProcessor
-from .FeatureProcessor import FeatureProcesser
+from .feature_processing import FeatureProcessor
 from .IntervalAnalyser import IntervalAnalyser
+from .Modeller import Modeller
+from .Sequencer import Sequencer
 from .Standardizer import Standardizer
